import java.sql.Connection;
import java.sql.CallableStatement;
import java.sql.SQLData;
import java.sql.SQLException;
import java.util.Map;
import oracle.jdbc.OracleTypes;
import oracle.sql.ARRAY;
import oracle.sql.ArrayDescriptor;

/** 
 * JavaWorldExample.
 *
 * For this (contrived) example, we will assume that Passenger vehicles represent the 
 * most basic type of vehicle features/functionality.  OffRoad vehicles are a superset
 * of Passenger vehicles in that they have an auxillary fuel tank.  Commercial vehicles 
 * are a superset of OffRoad vehicles in that they have a maintenance odometer.
 *
 * @author Carl Johnson <cjohnson@netjets.com>, Terry Camerlengo <tcamerlengo@hotmail.com>
 */

public class JavaWorldExample
{
    /** CONFIGURATION */
    private String host = "HOST";
    private int port = 1521;
    private String database = "DATABASE";
    private String username = "USERNAME";
    private String password = "PASSWORD";

    public JavaWorldExample()
        throws SQLException
    {
        java.sql.DriverManager.registerDriver(new oracle.jdbc.driver.OracleDriver());
    }

    public void run(String[] makes)
        throws SQLException
    {
        Connection connection = null;
        CallableStatement callableStatement = null;

        try
        {
            connection = java.sql.DriverManager.getConnection("jdbc:oracle:oci8:@(DESCRIPTION=(ADDRESS_LIST=(ADDRESS=(PROTOCOL=TCP)(HOST=" + host + ")(PORT=" + port + ")))(CONNECT_DATA=(SERVICE_NAME=" + database + ")))", username, password); 

            Map typeMap = connection.getTypeMap();

            typeMap.put(Odometer.ORACLE_OBJECT_NAME, Odometer.class);
            typeMap.put(FuelTank.ORACLE_OBJECT_NAME, FuelTank.class);
            typeMap.put(Vehicle.ORACLE_ARRAY_NAME, Vehicle[].class);
            typeMap.put(PassengerVehicle.ORACLE_OBJECT_NAME, PassengerVehicle.class);
            typeMap.put(OffRoadVehicle.ORACLE_OBJECT_NAME, OffRoadVehicle.class);
            typeMap.put(CommercialVehicle.ORACLE_OBJECT_NAME, CommercialVehicle.class);

            connection.setTypeMap(typeMap);

            callableStatement = connection.prepareCall("{? = call VEHICLE_MANAGER.SEARCH(?)}");

            ArrayDescriptor arrayDescriptor = ArrayDescriptor.createDescriptor("MAKE_TBL_T", connection);
            ARRAY array = new ARRAY(arrayDescriptor, connection, makes);

            callableStatement.setObject(2, array);
            
            callableStatement.registerOutParameter(1, OracleTypes.ARRAY, Vehicle.ORACLE_ARRAY_NAME); 
            callableStatement.execute();

            Object[] objects = (Object[])callableStatement.getArray(1).getArray();

            for (int i = 0; i < objects.length; i++)
                System.out.println(objects[i]);
        }
        finally
        {
            try
            {
                if (callableStatement != null)
                    callableStatement.close();
            }
            catch(SQLException e)
            {
                System.out.println("Exception encountered closing callable statement: " + e.getMessage());
            }

            try
            {
                if (connection != null)
                    connection.close();
            }
            catch(SQLException e)
            {
                System.out.println("Exception encountered closing connection: " + e.getMessage());
            }
        }
    }

    public static void main(String[] args)
    {
        if (args.length == 0)
        {
            System.out.println("usage: java JavaWorldExample [make]{1:n}");
            System.exit(1);
        }

        try
        {
            (new JavaWorldExample()).run(args);
        }
        catch(Exception e)
        {
            System.out.println("Failed to initialize/run JavaWorldExample!");

            e.printStackTrace();
        }
    }
}
